@@ -8,6 +8,18 @@ use SRU::Request::SearchRetrieve;
 use SRU::Request::Scan;
 use SRU::Utils qw( error );
 use SRU::Utils::XML qw( escape );
+use Scalar::Util qw(reftype);
+
+our %PARAMETERS = (
+    'explain' => 
+        [qw(version recordPacking stylesheet extraRequestData)],
+    'scan' => 
+        [qw(version scanClause responsePosition maximumTerms stylesheet 
+           extraRequestData)],
+    'searchRetrieve' => 
+        [qw(version query startRecord maximumRecords recordPacking recordSchema
+           recordXPath resultSetTTL sortKeys stylesheet extraRequestData)]
+);
 
 =head1 NAME
 
@@ -29,11 +41,9 @@ they contain.
 
 =head1 METHODS
 
-=head2 newFromURI()
+=head2 new( %query | $uri | $cgi | $env )
 
-newFromURI() is a factory method which you pass a complete SRU url. 
-newFromURI() will return an appropriate object for the type of request being 
-conducted:
+Create a new request object which is one of:
 
 =over 4
 
@@ -45,19 +55,45 @@ conducted:
 
 =back
 
+One can pass query parameters as hash, as URL, as L<URI>, as L<CGI> object or
+as L<PSGI> request.
+
 If the request is not formatted properly the call will return undef. 
 The error encountered should be available in $SRU::Error.
 
 =cut
 
-sub newFromURI {
-    my ($class,$uri) = @_;
+sub new {
+    my $class = shift;
 
-    ## be nice and try to turn a string into a URI if necessary
-    if ( ! UNIVERSAL::isa( $uri, 'URI' ) ) { $uri = URI->new($uri); }
-    return error( "invalid uri: $uri" ) if ! UNIVERSAL::isa( $uri, 'URI' ); 
+    my %query;
+
+    if ( @_ % 2 ) {
+        my $q = shift;
+
+        if ( UNIVERSAL::isa( $q, 'CGI' ) ) {
+            ## we must have ampersands between query string params, but lets
+            ## make sure we don't screw anybody else up
+            my $saved = $CGI::USE_PARAM_SEMICOLONS; 
+            $CGI::USE_PARAM_SEMICOLONS = 0;
+            $q = $q->self_url;
+            $CGI::USE_PARAM_SEMICOLONS = $saved;
+        } elsif ( (reftype $q // '') eq 'HASH' ) {
+            $q = "http://example.org/?" . $q->{QUERY_STRING};
+        }
+            
+        if ( ! UNIVERSAL::isa( $q, 'URI' ) ) { 
+            $q = URI->new($q);
+        }
+        if ( UNIVERSAL::isa( $q, 'URI' ) ) {
+            %query = $q->query_form;
+        } else {
+            return error( "invalid uri: $q" ) 
+        }
+    } else {
+        %query = @_;
+    }
 
-    my %query     = $uri->query_form();
     my $operation = $query{operation} || 'explain';
 
     my $request;
@@ -73,32 +109,19 @@ sub newFromURI {
     }
 
     return $request;
+
 }
 
-=head2 newFromCGI()
+=head2 newFromURI
 
-A factory method for creating a request object from a CGI object.
+=head2 newFromCGI
 
-    my $cgi = CGI->new();
-    my $request = SRU::Request->newFromCGI( $cgi );
+Deprecated aliases for C<new>.
 
 =cut
 
-sub newFromCGI {
-    my ($class,$cgi) = @_;
-
-    ## We want either an actual CGI object
-    return error( "invalid CGI object" ) unless UNIVERSAL::isa( $cgi, 'CGI' );
-
-    ## we must have ampersands between query string params, but lets
-    ## make sure we don't screw anybody else up
-    my $saved = $CGI::USE_PARAM_SEMICOLONS; 
-    $CGI::USE_PARAM_SEMICOLONS = 0;
-    my $url = $cgi->self_url();
-    $CGI::USE_PARAM_SEMICOLONS = $saved;
-
-    return $class->newFromURI( $url );
-}
+*newFromURI = *new;
+*newFromCGI = *new;
 
 =head2 asXML()
 
@@ -137,6 +160,31 @@ sub asXML {
     return $xml;
 }
 
+=head2 asURI( [ $base ] )
+
+Creates a L<URI> of this request. The optional C<base> URL, provided as
+string or as L<URI>, is set to C<http://localhost/> by default.
+
+=cut
+
+sub asURI {
+    my ($self, $base) = @_;
+
+    my $uri = URI->new($base // "http://localhost/");
+    my %query = $uri->query_form;
+
+    $query{operation} = $self->type;
+    
+    no strict 'refs';
+    foreach (@{ $PARAMETERS{ $self->type } }) {
+        $query{$_} = $self->$_ if defined $self->$_;
+    }
+
+    $uri->query_form( \%query );
+    return $uri;
+}
+
+
 =head2 type()
 
 Returns 'searchRetrieve', 'scan' or 'explain' depending on what type of
